import java.nio.IntBuffer;
import org.lwjgl.opengl.GL20;
import org.lwjgl.opengl.ARBShaderObjects;
import org.lwjgl.opengl.GLContext;
import org.lwjgl.opengl.GL11;
import java.io.Reader;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.File;
import java.net.URISyntaxException;
import java.net.URI;
import org.lwjgl.BufferUtils;
import java.util.ArrayList;
import java.nio.FloatBuffer;

public class Shader
{
    private int vertex_shader;
    private int fragment_shader;
    private int program;
    private FloatBuffer matrix;
    private boolean disable_error_msg;
    private boolean debug;
    private ArrayList value_ids;
    private boolean pre_opengl20;
    
    Shader(final String path) {
        this.vertex_shader = 0;
        this.fragment_shader = 0;
        this.program = 0;
        this.matrix = BufferUtils.createFloatBuffer(16);
        this.disable_error_msg = true;
        this.debug = false;
        this.value_ids = new ArrayList();
        this.pre_opengl20 = false;
        final String vert_path = path + ".vert";
        final String frag_path = path + ".frag";
        String line = "";
        String vertex_shader_source = "";
        URI uri = null;
        try {
            uri = new URI("file://" + Shader.class.getProtectionDomain().getCodeSource().getLocation().getPath());
        }
        catch (final URISyntaxException e1) {
            throw new RuntimeException("URI fault");
        }
        final File f = new File(uri);
        try {
            final BufferedReader reader = new BufferedReader(new FileReader(f.getParent() + vert_path));
            while ((line = reader.readLine()) != null) {
                vertex_shader_source = vertex_shader_source + line + "\n";
            }
        }
        catch (final Exception e2) {
            throw new RuntimeException("Couldnt load shader! Was looking for: " + f.getParent() + vert_path + " \nReason: " + e2);
        }
        final String version = GL11.glGetString(7938);
        if (Integer.valueOf(version.substring(0, 1)) < 2) {
            this.pre_opengl20 = true;
            if (!GLContext.getCapabilities().GL_ARB_shader_objects) {
                throw new RuntimeException("Pre OpenGL 2.0 without GL_ARB_shader_objects support.");
            }
            if (!GLContext.getCapabilities().GL_ARB_vertex_shader) {
                throw new RuntimeException("Pre OpenGL 2.0 without GL_ARB_vertex_shader support.");
            }
            if (!GLContext.getCapabilities().GL_ARB_fragment_shader) {
                throw new RuntimeException("Pre OpenGL 2.0 without GL_ARB_vertex_shader support.");
            }
        }
        if (!this.pre_opengl20) {
            this.vertex_shader = this.compile(35633, vertex_shader_source);
        }
        else {
            this.vertex_shader = this.compilePre20(35633, vertex_shader_source);
        }
        if (this.vertex_shader == 0) {
            System.out.println("Error Vertex Shader!");
        }
        String fragment_shader_source = "";
        try {
            final BufferedReader reader = new BufferedReader(new FileReader(f.getParent() + frag_path));
            while ((line = reader.readLine()) != null) {
                fragment_shader_source = fragment_shader_source + line + "\n";
            }
        }
        catch (final Exception e3) {
            e3.printStackTrace();
        }
        if (!this.pre_opengl20) {
            this.fragment_shader = this.compile(35632, fragment_shader_source);
        }
        else {
            this.fragment_shader = this.compilePre20(35632, fragment_shader_source);
        }
        if (this.fragment_shader == 0) {
            System.out.println("Error Fragment Shader!");
        }
        if (!this.pre_opengl20) {
            this.link(path);
        }
        else {
            this.linkPre20(path);
        }
    }
    
    private int compilePre20(final int type, final String shader_source) {
        final int shader = ARBShaderObjects.glCreateShaderObjectARB(type);
        if (shader == 0) {
            System.out.println("Impossible to create shader object!");
            return 0;
        }
        ARBShaderObjects.glShaderSourceARB(shader, (CharSequence)shader_source);
        ARBShaderObjects.glCompileShaderARB(shader);
        return shader;
    }
    
    private int compile(final int type, final String shader_source) {
        final int shader = GL20.glCreateShader(type);
        if (shader == 0) {
            System.out.println("Impossible to create shader object!");
            return 0;
        }
        GL20.glShaderSource(shader, (CharSequence)shader_source);
        GL20.glCompileShader(shader);
        return shader;
    }
    
    private void linkPre20(final String source) {
        ARBShaderObjects.glAttachObjectARB(this.program = ARBShaderObjects.glCreateProgramObjectARB(), this.vertex_shader);
        ARBShaderObjects.glAttachObjectARB(this.program, this.fragment_shader);
        ARBShaderObjects.glLinkProgramARB(this.program);
        final IntBuffer result = BufferUtils.createIntBuffer(1);
        ARBShaderObjects.glGetObjectParameterARB(this.program, 35714, result);
        if (result.get(0) != 1) {
            throw new RuntimeException("Shader compile error! \n\nin: " + source + "\n\n" + ARBShaderObjects.glGetInfoLogARB(this.program, 5000));
        }
        System.out.println(ARBShaderObjects.glGetInfoLogARB(this.program, 5000));
    }
    
    private void link(final String source) {
        GL20.glAttachShader(this.program = GL20.glCreateProgram(), this.vertex_shader);
        GL20.glAttachShader(this.program, this.fragment_shader);
        GL20.glLinkProgram(this.program);
        final IntBuffer result = BufferUtils.createIntBuffer(1);
        GL20.glGetProgram(this.program, 35714, result);
        if (result.get(0) != 1) {
            throw new RuntimeException("Shader compile error! \n\nin: " + source + "\n\n" + GL20.glGetProgramInfoLog(this.program, 5000));
        }
        System.out.println(GL20.glGetProgramInfoLog(this.program, 5000));
    }
    
    public void bind() {
        if (!this.pre_opengl20) {
            GL20.glUseProgram(this.program);
        }
        else {
            ARBShaderObjects.glUseProgramObjectARB(this.program);
        }
    }
    
    public void unbind() {
        if (!this.pre_opengl20) {
            GL20.glUseProgram(0);
        }
        else {
            ARBShaderObjects.glUseProgramObjectARB(0);
        }
    }
    
    public int initValue1i(final String name) {
        int location = 0;
        if (!this.pre_opengl20) {
            location = GL20.glGetUniformLocation(this.program, (CharSequence)name);
        }
        else {
            location = ARBShaderObjects.glGetUniformLocationARB(this.program, (CharSequence)name);
        }
        if (!this.disable_error_msg && location == -1) {
            System.out.println("Error binding shader int.");
        }
        this.value_ids.add(location);
        if (this.debug) {
            System.out.println("Added: " + name + " at " + (this.value_ids.size() - 1) + " location: " + location);
        }
        return this.value_ids.size() - 1;
    }
    
    public void setValue1i(final int position_in_location_array, final int value) {
        if (!this.pre_opengl20) {
            GL20.glUniform1i((int)this.value_ids.get(position_in_location_array), value);
        }
        else {
            ARBShaderObjects.glUniform1iARB((int)this.value_ids.get(position_in_location_array), value);
        }
    }
    
    public int initValue1f(final String name) {
        int location = 0;
        if (!this.pre_opengl20) {
            location = GL20.glGetUniformLocation(this.program, (CharSequence)name);
        }
        else {
            location = ARBShaderObjects.glGetUniformLocationARB(this.program, (CharSequence)name);
        }
        if (!this.disable_error_msg && location == -1) {
            System.out.println("Error binding shader float.");
        }
        this.value_ids.add(location);
        if (this.debug) {
            System.out.println("Added: " + name + " at " + (this.value_ids.size() - 1) + " location: " + location);
        }
        return this.value_ids.size() - 1;
    }
    
    public void setValue1f(final int position_in_location_array, final float value) {
        if (!this.pre_opengl20) {
            GL20.glUniform1f((int)this.value_ids.get(position_in_location_array), value);
        }
        else {
            ARBShaderObjects.glUniform1fARB((int)this.value_ids.get(position_in_location_array), value);
        }
    }
    
    public int initValueVec3f(final String name) {
        int location = 0;
        if (!this.pre_opengl20) {
            location = GL20.glGetUniformLocation(this.program, (CharSequence)name);
        }
        else {
            location = ARBShaderObjects.glGetUniformLocationARB(this.program, (CharSequence)name);
        }
        if (!this.disable_error_msg && location == -1) {
            System.out.println("Error binding shader vec3.");
        }
        this.value_ids.add(location);
        if (this.debug) {
            System.out.println("Added: " + name + " at " + (this.value_ids.size() - 1) + " location: " + location);
        }
        return this.value_ids.size() - 1;
    }
    
    public void setValueVec3f(final int position_in_location_array, final float x, final float y, final float z) {
        if (!this.pre_opengl20) {
            GL20.glUniform3f((int)this.value_ids.get(position_in_location_array), x, y, z);
        }
        else {
            ARBShaderObjects.glUniform3fARB((int)this.value_ids.get(position_in_location_array), x, y, z);
        }
    }
    
    public int initValueMat4f(final String name) {
        int location = 0;
        if (!this.pre_opengl20) {
            location = GL20.glGetUniformLocation(this.program, (CharSequence)name);
        }
        else {
            location = ARBShaderObjects.glGetUniformLocationARB(this.program, (CharSequence)name);
        }
        if (!this.disable_error_msg && location == -1) {
            System.out.println("Error binding shader matrix.");
        }
        this.value_ids.add(location);
        if (this.debug) {
            System.out.println("Added: " + name + " at " + (this.value_ids.size() - 1) + " location: " + location);
        }
        return this.value_ids.size() - 1;
    }
    
    public void setValueMat4f(final int position_in_location_array, final float[] value) {
        this.matrix.clear();
        for (int i = 0; i < 16; ++i) {
            this.matrix.put(value[i]);
        }
        this.matrix.flip();
        if (!this.pre_opengl20) {
            GL20.glUniformMatrix4((int)this.value_ids.get(position_in_location_array), true, this.matrix);
        }
        else {
            ARBShaderObjects.glUniformMatrix4ARB((int)this.value_ids.get(position_in_location_array), true, this.matrix);
        }
    }
}
